<?php

namespace Ardent\Game;

use Ardent\Game\dto\Color;

class Noise
{
    private readonly PerlinNoise $noise;

    public function __construct(
        ?int $seed = null,
    )
    {
        $this->noise = new PerlinNoise($seed);
    }

    private function getTransitions(): array
    {
        $deepwater = new Color('#006eb4');
        $water = new Color(0, 156, 244);
        $sand = new Color(238, 226, 104);
        $grass = new Color(25, 171, 64);
        $mountain = new Color(96, 93, 100);
        $snow = new Color(255, 255, 255);

        return [
            [-1.0, $deepwater],
            [-0.9, $water],
            [0.0, $sand],
            [0.8, $grass],
            [0.997, $mountain],
            [1.0, $snow],
        ];
    }

    public function generate(int $x, int $y): Color
    {
        $rand = $this->noise->random2D($x, $y, 150);
        $transitions = $this->getTransitions();

        for ($i = 0; $i < count($transitions) - 1; $i++) {
            $transition = $transitions[$i];
            $nextTransition = $transitions[$i + 1];
            if ($rand >= $transition[0] && $rand < $nextTransition[0]) {
                return $this->lerpTerrain($transition[1], $nextTransition[1], $transition[0], $nextTransition[0], $rand);
            }
        }
    }

    public function lerpTerrain(Color $color1, Color $color2, float $begin, float $end, float $value): Color
    {
        return $color1->lerp($color2, ($value - $begin) * (1.0 / ($end - $begin)));
    }
}